import pytest
import pyperclip
from typer.testing import CliRunner
from ..main import app
from ..lib.curl_parser import CurlParser

runner = CliRunner()

def test_convert_grab():
    curl_cmd = """curl 'https://api.example.com/data'"""
    pyperclip.copy(curl_cmd)
    
    result = runner.invoke(app, ["--framework", "grab"])
    assert result.exit_code == 0
    assert "has been copied to clipboard" in result.stdout

def test_convert_context():
    curl_cmd = """curl 'https://api.example.com/data'"""
    pyperclip.copy(curl_cmd)
    
    result = runner.invoke(app, ["--framework", "context"])
    assert result.exit_code == 0
    assert "has been copied to clipboard" in result.stdout

def test_show_output():
    curl_cmd = """curl 'https://api.example.com/data'"""
    pyperclip.copy(curl_cmd)
    
    result = runner.invoke(app, ["--framework", "grab", "--show"])
    assert result.exit_code == 0
    assert "self.g.go" in result.stdout

def test_invalid_curl():
    pyperclip.copy("not a curl command")
    
    result = runner.invoke(app, ["--framework", "grab"])
    assert result.exit_code == 1
    assert "Invalid curl command" in result.stdout

def test_version():
    result = runner.invoke(app, ["--version"])
    assert result.exit_code == 0
    assert "curl-to-context version:" in result.stdout

def test_convert_interactive(monkeypatch):
    # Mock user inputs
    inputs = iter(['grab', ''])  # Framework selection and curl command confirmation
    monkeypatch.setattr('builtins.input', lambda _: next(inputs))
    
    # Mock clipboard content
    curl_cmd = """curl 'https://api.example.com/data'"""
    pyperclip.copy(curl_cmd)
    
    result = runner.invoke(app)
    assert result.exit_code == 0
    assert "has been copied to clipboard" in result.stdout 


import pytest
from ..lib.grab import GrabCodeWriter

def test_simple_get():
    parsed = {
        'method': 'get',
        'url': 'https://api.example.com/data',
        'data': {},
        'headers': {},
        'cookies': {}
    }
    
    writer = GrabCodeWriter(parsed)
    code = writer.generate_code()
    
    assert "self.g.go('https://api.example.com/data')" in code

def test_with_headers():
    parsed = {
        'method': 'get',
        'url': 'https://api.example.com/data',
        'data': {},
        'headers': {'accept': 'application/json'},
        'cookies': {}
    }
    
    writer = GrabCodeWriter(parsed)
    code = writer.generate_code()
    
    assert 'self.g.setup(headers=' in code
    assert '"accept": "application/json"' in code

def test_with_cookies():
    parsed = {
        'method': 'get',
        'url': 'https://api.example.com/data',
        'data': {},
        'headers': {},
        'cookies': {'session': 'abc123'}
    }
    
    writer = GrabCodeWriter(parsed)
    code = writer.generate_code()
    
    assert 'self.g.setup(cookies=' in code
    assert '"session": "abc123"' in code

def test_post_request():
    parsed = {
        'method': 'post',
        'url': 'https://api.example.com/data',
        'data': {'key': 'value'},
        'data_as_json': True,
        'headers': {},
        'cookies': {}
    }
    
    writer = GrabCodeWriter(parsed)
    code = writer.generate_code()
    
    assert 'post_params' in code
    assert 'self.g.go' in code
    assert 'json=post_params' in code



import typer
import pyperclip
from rich import print
from rich.panel import Panel
from rich.syntax import Syntax
from rich.traceback import install
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.prompt import Prompt
from typing import Optional
from enum import Enum
from prompt_toolkit import prompt
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.styles import Style
from catppuccin.extras.pygments import MochaStyle
import importlib.metadata
import time

# Install rich traceback handler
install(show_locals=True)

from .lib.curl_parser import CurlParser
from .lib.grab import GrabCodeWriter
from .lib.context import ContextCodeWriter

app = typer.Typer(
    help="Convert cURL commands to Python code for Grab/Context frameworks",
    add_completion=True,
    no_args_is_help=True
)

def get_version():
    """Get package version from pyproject.toml."""
    try:
        return importlib.metadata.version("curl-to-context")
    except importlib.metadata.PackageNotFoundError:
        return "unknown"

class Framework(str, Enum):
    GRAB = "grab"
    CONTEXT = "context"

def get_framework() -> Framework:
    """Interactive framework selection with autocomplete and syntax highlighting."""
    frameworks = [f.value for f in Framework]
    
    # Create styled completer
    style = Style.from_dict({
        'completion-menu.completion': 'bg:#1e1e2e #cdd6f4',
        'completion-menu.completion.current': 'bg:#313244 #cdd6f4',
        'completion-menu.meta.completion': 'bg:#1e1e2e #cdd6f4',
        'completion-menu.meta.completion.current': 'bg:#313244 #cdd6f4',
        'completion.grab': 'bg:#1e1e2e #89b4fa',  # Catppuccin Blue
        'completion.context': 'bg:#1e1e2e #f9e2af',  # Catppuccin Yellow
    })
    
    framework_completer = WordCompleter(
        frameworks,
        ignore_case=True,
        meta_dict={
            'grab': 'Grab framework',
            'context': 'Context framework'
        },
        style_dict={
            'grab': 'class:completion.grab',
            'context': 'class:completion.context'
        }
    )
    
    while True:
        try:
            selected = Prompt.ask(
                "[cyan]Choose framework[/cyan]",
                choices=frameworks,
                default=Framework.GRAB.value
            )
            return Framework(selected.lower())
        except ValueError:
            print("[red]Invalid framework. Please choose 'grab' or 'context'[/red]")

def display_code(code: str, language: str = "python"):
    """Display code with syntax highlighting using Catppuccin theme."""
    syntax = Syntax(
        code,
        language,
        theme=MochaStyle,
        line_numbers=True,
        word_wrap=True,
    )
    panel = Panel(
        syntax,
        expand=False,
        border_style="#89dceb",  # Catppuccin Sky
        title=f"Generated {language.capitalize()} Code",
        title_align="left",
    )
    print(panel)

def get_curl_command() -> str:
    """Get curl command from clipboard with loading animation."""
    with Progress(
        SpinnerColumn(style="cyan"),
        TextColumn("[cyan]Waiting for curl command...[/cyan]"),
        transient=True,
    ) as progress:
        progress.add_task("waiting")
        input("Copy curl command to clipboard and press Enter...")
        
    curl_command = pyperclip.paste()
    
    if not curl_command.strip().startswith('curl '):
        raise ValueError("Invalid curl command. Please copy a valid curl command to clipboard.")
    
    return curl_command

def form_code(parsed_curl: dict, framework: Framework) -> str:
    """Generate framework-specific code from parsed curl command."""
    if framework == Framework.GRAB:
        writer = GrabCodeWriter(parsed_curl)
    else:
        writer = ContextCodeWriter(parsed_curl)
    
    code = writer.generate_code()
    
    # Syntax highlight the generated code
    return str(Syntax(code, "python", theme="monokai"))

def version_callback(value: bool):
    """Callback for --version flag."""
    if value:
        version = get_version()
        print(f"curl-to-context version: {version}")
        raise typer.Exit()

@app.callback(invoke_without_command=True)
def main(
    ctx: typer.Context,
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-V",
        help="Show the application's version and exit.",
        callback=version_callback,
        is_eager=True,
    )
):
    """Convert cURL commands to Python code for Grab/Context frameworks."""
    if ctx.invoked_subcommand is None:
        # If no subcommand provided, run convert without args
        ctx.invoke(convert)

@app.command()
def convert(
    framework: Optional[Framework] = None,
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-v", 
        help="Display the converted code in terminal"
    )
):
    """Convert a cURL command from clipboard to Python code."""
    try:
        if not framework:
            framework = get_framework()
            print()  # Add spacing
        
        curl_command = get_curl_command()
        parsed_curl = CurlParser.parse_curl(curl_command)
        python_code = form_code(parsed_curl, framework=framework)
        
        # Copy plain text to clipboard
        pyperclip.copy(python_code)
        print(f"[green]✓[/green] Converted code has been copied to clipboard!")
        
        # If verbose, display with syntax highlighting
        if verbose:
            display_code(python_code)
            
    except Exception as e:
        print(f"[red]Error:[/red] {str(e)}")
        raise typer.Exit(1)

def c2c():
    """Wrapper for context framework."""
    app(["convert", "-f", "context"])

def c2g():
    """Wrapper for grab framework."""
    app(["convert", "-f", "grab"])

def curl2context():
    """Wrapper for context framework."""
    app(["convert", "-f", "context"])

def curl2grab():
    """Wrapper for grab framework."""
    app(["convert", "-f", "grab"])

if __name__ == "__main__":
    app()


import re
import json
import shlex
from argparse import ArgumentParser
from collections import OrderedDict
from typing import Dict, Any, List, Tuple, Optional
from urllib.parse import unquote_plus, parse_qsl, urlsplit
from http.cookies import SimpleCookie

class CurlParser:
    """Parser for curl commands with modern Python features."""
    
    @staticmethod
    def parse_curl(curl_command: str) -> Dict[str, Any]:
        """Parse curl command into structured data."""
        if not curl_command.strip().startswith('curl '):
            raise ValueError("Not a valid curl command")
            
        parser = ArgumentParser()
        parser.add_argument('command')
        parser.add_argument('url', nargs='?')
        parser.add_argument('-d', '--data')
        parser.add_argument('-b', '--data-binary', '--data-raw', default=None)
        parser.add_argument('-X', default='')
        parser.add_argument('-H', '--header', action='append', default=[])
        
        try:
            parsed_args = parser.parse_args(shlex.split(curl_command))
            if not parsed_args.url:
                raise ValueError("URL is required")
        except Exception as e:
            if isinstance(e, SystemExit):
                raise ValueError("Invalid curl command format")
            raise ValueError(str(e))

        return CurlParser._process_parsed_args(parsed_args)
    
    @staticmethod
    def _process_parsed_args(parsed_args) -> Dict[str, Any]:
        """Process parsed arguments into structured data."""
        post_data = parsed_args.data or parsed_args.data_binary
        
        if post_data:
            return CurlParser._handle_post_request(post_data, parsed_args)
        else:
            return CurlParser._handle_get_request(parsed_args)

    @staticmethod
    def _handle_post_request(post_data: str, parsed_args) -> Dict[str, Any]:
        """Handle POST request parsing."""
        post_data = post_data.strip().lstrip('$')
        
        try:
            # First try to parse as JSON
            try:
                post_data_dict = json.loads(post_data)
                data_as_json = True
                ordered_post_data = None
            except json.JSONDecodeError:
                # If not JSON, try query string
                ordered_post_data = parse_qsl(post_data)
                post_data_dict = dict(ordered_post_data)
                data_as_json = False
        except Exception:
            # If both fail, try JavaScript object notation
            data_as_json = True
            post_data_dict = CurlParser._eval_js_object(post_data)
            ordered_post_data = None

        return {
            'method': 'post',
            'url': parsed_args.url,
            'data': post_data_dict,
            'ordered_data': ordered_post_data,
            'data_as_json': data_as_json,
            **CurlParser._parse_headers_and_cookies(parsed_args.header)
        }

    @staticmethod
    def _handle_get_request(parsed_args) -> Dict[str, Any]:
        """Handle GET request parsing."""
        parsed_url = urlsplit(parsed_args.url)
        ordered_data_dict = parse_qsl(parsed_url.query)
        
        return {
            'method': 'get',
            'url': f'{parsed_url.scheme}://{parsed_url.netloc}{parsed_url.path}',
            'data': dict(ordered_data_dict),
            'ordered_data': ordered_data_dict,
            'data_as_json': False,
            **CurlParser._parse_headers_and_cookies(parsed_args.header)
        }

    @staticmethod
    def _parse_headers_and_cookies(headers: List[str]) -> Dict[str, Dict[str, str]]:
        """Parse headers and cookies from curl command."""
        cookie_dict = OrderedDict()
        headers_dict = OrderedDict()

        for header in headers:
            key, value = header.split(":", 1)
            
            if key.lower() == 'cookie':
                cookie = SimpleCookie(value)
                for k in cookie:
                    try:
                        cookie_dict[k] = unquote_plus(cookie[k].value)
                    except Exception:
                        cookie_dict[k] = cookie[k].value
            else:
                headers_dict[key] = value.strip()

        return {
            'headers': headers_dict,
            'cookies': cookie_dict
        }

    @staticmethod
    def _eval_js_object(data: str) -> Dict[str, Any]:
        """Evaluate JSON-like JavaScript code object."""
        loops_left = 200
        while 'Array(' in data and loops_left > 0:
            data = re.sub(r'(?:new\s+)?Array\(([^\)]*)\)', r'[\g<1>]', data)
            loops_left -= 1
            
        if 'Array(' in data:
            raise ValueError("Couldn't convert Array()")

        data = re.sub(r"([a-zA-Z]\w+)\s*:", r"'\g<1>':", data)
        data = data.replace("'", '"')

        for apos in re.findall('[a-zA-Z]\"[a-zA-Z]', data):
            data = data.replace(apos, apos.replace('"', "'"))

        return json.loads(data)



from typing import Dict, Any
import json

class ContextCodeWriter:
    """Generates Context framework code from parsed curl commands."""
    
    def __init__(self, parsed_command: Dict[str, Any]):
        self.parsed = parsed_command
        
    def generate_code(self) -> str:
        """Generate Context framework code from parsed curl command."""
        code_parts = [
            self._generate_headers(),
            self._generate_cookies(),
            self._generate_request()
        ]
        
        return '\n\n'.join([p for p in code_parts if p])

    def _generate_headers(self) -> str:
        """Generate code for headers setup."""
        if not self.parsed.get('headers'):
            return ''
            
        headers = json.dumps(self.parsed['headers'], indent=4)
        return f'self.context.headers.update({headers})'

    def _generate_cookies(self) -> str:
        """Generate code for cookies setup."""
        if not self.parsed.get('cookies'):
            return ''
            
        cookies = json.dumps(self.parsed['cookies'], indent=4)
        return f'self.context.cookies.update({cookies})'

    def _generate_request(self) -> str:
        """Generate the actual request code."""
        url = self.parsed['url']
        method = self.parsed['method'].upper()
        
        if not self.parsed.get('data'):
            return f'response = self.context.{method}("{url}")'
            
        data = json.dumps(self.parsed['data'], indent=4)
        data_type = 'json' if self.parsed['data_as_json'] else 'data'
        
        return f'response = self.context.{method}("{url}", {data_type}={data})'



from typing import Dict, Any
import json
from collections import OrderedDict

class GrabCodeWriter:
    """Generates Grab framework code from parsed curl commands."""
    
    COMMON_HEADERS = [
        'accept-encoding',
        'accept-language',
        'accept',
        'connection',
        'user-agent'
    ]

    def __init__(self, parsed_command: Dict[str, Any]):
        self.parsed = parsed_command
        
    def generate_code(self) -> str:
        """Generate Grab framework code from parsed curl command."""
        code_parts = [
            self._generate_headers(),
            self._generate_cookies(),
            self._generate_params(),
            self._generate_request()
        ]
        
        return '\n\n'.join([p for p in code_parts if p])

    def _generate_headers(self) -> str:
        """Generate code for headers setup."""
        if not self.parsed.get('headers'):
            return ''
            
        headers_dict = self._dict_to_python(
            self.parsed['headers'],
            'headers',
            mark_common=True
        )
        return f'self.g.setup({headers_dict})'

    def _generate_cookies(self) -> str:
        """Generate code for cookies setup."""
        if not self.parsed.get('cookies'):
            return ''
            
        cookies_dict = self._dict_to_python(
            self.parsed['cookies'],
            'cookies'
        )
        return f'self.g.setup({cookies_dict})'

    def _generate_params(self) -> str:
        """Generate code for request parameters."""
        if not self.parsed.get('data'):
            return ''
            
        params_name = f"{self.parsed['method']}_params"
        return self._dict_to_python(
            self.parsed['data'],
            params_name
        )

    def _generate_request(self) -> str:
        """Generate the actual request code."""
        url = self.parsed['url']
        method = self.parsed['method']
        
        if not self.parsed.get('data'):
            return f"self.g.go('{url}')"
            
        params_name = f"{method}_params"
        param_type = 'json' if self.parsed['data_as_json'] else 'post'
        
        if method == 'get':
            return f"self.g.go('{url}', params={params_name})"
        else:
            return f"self.g.go('{url}', {param_type}={params_name})"

    def _dict_to_python(self, data: Dict, var_name: str, mark_common: bool = False) -> str:
        """Convert dictionary to Python code string."""
        dict_str = json.dumps(data, indent=4)
        
        if mark_common:
            lines = []
            for line in dict_str.split('\n'):
                if any(h in line.lower() for h in self.COMMON_HEADERS):
                    line += ' # should not be necessary'
                lines.append(line)
            dict_str = '\n'.join(lines)
            
        return f"{var_name}={dict_str}"


import pytest
from ..lib.curl_parser import CurlParser

def test_parse_simple_get():
    curl = """curl 'https://api.example.com/data'"""
    result = CurlParser.parse_curl(curl)
    
    assert result['method'] == 'get'
    assert result['url'] == 'https://api.example.com/data'
    assert not result['data']
    assert not result['headers']
    assert not result['cookies']

def test_parse_with_headers():
    curl = """curl 'https://api.example.com/data' -H 'accept: application/json' -H 'user-agent: Mozilla/5.0'"""
    result = CurlParser.parse_curl(curl)
    
    assert result['headers'] == {
        'accept': 'application/json',
        'user-agent': 'Mozilla/5.0'
    }

def test_parse_with_cookies():
    curl = """curl 'https://api.example.com/data' -H 'cookie: session=abc123; user=john'"""
    result = CurlParser.parse_curl(curl)
    
    assert result['cookies'] == {
        'session': 'abc123',
        'user': 'john'
    }

def test_parse_post_data():
    curl = """curl 'https://api.example.com/data' -d '{"key":"value"}'"""
    result = CurlParser.parse_curl(curl)
    
    assert result['method'] == 'post'
    assert result['data'] == {'key': 'value'}
    assert result['data_as_json'] == True

def test_invalid_curl():
    with pytest.raises(ValueError):
        CurlParser.parse_curl("not a curl command")
